BEGIN
    DBMS_OUTPUT.PUT_LINE (
       q'[What's a quote among friends?]');
END;
/
BEGIN
    DBMS_OUTPUT.PUT_LINE (
       q'!What's a quote among friends?!');
END;
/

/* Traditional doubling up */

BEGIN
   DBMS_OUTPUT.put_line ('That''s a really funny ''joke''.');
END;
/

/* Proper pairing of open and close parens. */

BEGIN
   DBMS_OUTPUT.put_line (q'(That's a really funny 'joke'.)');
END;
/

/* Can't use a "paired" symbol as open and close */

BEGIN
   DBMS_OUTPUT.put_line (q'(That''s a really funny ''joke''.(');
END;
/

BEGIN
   DBMS_OUTPUT.put_line (q'#That's a really funny 'joke'.#');
END;
/

/* Improper use of two different non-paired characters. */

BEGIN
   DBMS_OUTPUT.put_line (q'#That's a really funny 'joke'.$');
END;
/

/* Works in SQL, too! */

SELECT q''All the President's men''
 FROM  DUAL;

/* Use a quote delimiter that unlikely to appear in the literal; the quote
   delimiter cannot be a space, tab or return: */

SELECT q'#All the President's men#'
 FROM  DUAL;

/* Cannot be a space */

SELECT q' All the President's men '
 FROM  DUAL;

/*ORA-00911: invalid character*/

/* Can be brackets, but an initial opening bracket must be matched by the
   corresponding closing one: */

SELECT q'>All the President's men>'
 FROM  DUAL;

SELECT q'>All the President's men<'
 FROM  DUAL;

SELECT q'<All the President's men<'
 FROM  DUAL;

/*ORA-01756: quoted string not properly terminated */

/* q notation can be used in SQL and PL/SQL; the quote delimiter itself can appears within
   as long as it's not followed by a single quote: */

BEGIN
   dbms_output.put_line (q'PAll the President's menP');
END;
/

/* q notation can also be used in dynamic SQL; | can be used as the quote delimiter as long
   as it's not followed by a single quote: */

DECLARE
   l_men_or_women VARCHAR2(5) := 'men';
BEGIN
EXECUTE IMMEDIATE
     q'|BEGIN
           dbms_output.put_line('All the President''s |'||l_men_or_women||q'|');
        END;
      |';
END;
/

/* Here's usages of several valid pairings */

BEGIN
   DBMS_OUTPUT.put_line (q'['Hello,' said the man, who didn't like goodbyes.]');
END;
/

BEGIN
   DBMS_OUTPUT.put_line (q'{'Hello,' said the man, who didn't like goodbyes.}');
END;
/

BEGIN
   DBMS_OUTPUT.put_line (Q'('Hello,' said the man, who didn't like goodbyes.)');
END;
/

BEGIN
   DBMS_OUTPUT.put_line (q'<'Hello,' said the man, who didn't like goodbyes.>');
END;
/
