create or replace type strings_nt is
   table of varchar2(1000)
/

create or replace package authors_pkg 
is
   steven_authors strings_nt := strings_nt(
      'ROBIN HOBB',
      'ROBERT HARRIS',
      'ROBERT HARRIS',
      'DAVID BRIN',
      'DAVID BRIN',
      'SHERI S. TEPPER',
      'CHRISTOPHER ALEXANDER'
   );
   
   veva_authors strings_nt := strings_nt(
      'ROBIN HOBB',
      'SHERI S. TEPPER',
      'ANNE MCCAFFREY',
      'DAVID BRIN',
      'DAVID BRIN'
   );
   
   procedure show_authors (
      title_in    in  varchar2,
      authors_in  in  strings_nt
   );
end;
/

create or replace package body authors_pkg 
is
   procedure show_authors (
      title_in    in  varchar2,
      authors_in  in  strings_nt
   ) 
   is
   begin
      DBMS_OUTPUT.put_line(title_in);
      for indx in 1..authors_in.COUNT loop
         DBMS_OUTPUT.put_line(indx
                              || ' = '
                              || authors_in(indx));
      end loop;
   end show_authors;
end;
/

declare
   our_authors strings_nt;
begin
   our_authors  := authors_pkg.steven_authors 
                      multiset union 
                   authors_pkg.veva_authors;
                   
   authors_pkg.show_authors('Steven and Veva', our_authors);

   DBMS_OUTPUT.put_line('Union inside SQL');
   
   for rec in (
      select COLUMN_VALUE
        from table ( authors_pkg.veva_authors 
                     multiset union authors_pkg.steven_authors )
       order by COLUMN_VALUE
   ) loop
      DBMS_OUTPUT.put_line(rec.COLUMN_VALUE);
   end loop;

   our_authors  := authors_pkg.steven_authors 
                      multiset union distinct authors_pkg.veva_authors;
                      
   authors_pkg.show_authors('Steven then Veva with DISTINCT', our_authors);
end;
/