/* Use IS JSON check constraint for your JSON columns. */

CREATE TABLE j_purchaseorder
(
   id            VARCHAR2 (32) NOT NULL PRIMARY KEY,
   date_loaded   TIMESTAMP (6) WITH TIME ZONE,
   po_document   VARCHAR2 (23767) 
      CONSTRAINT ensure_json CHECK (po_document IS JSON))
/


SELECT po.po_document.Requestor FROM j_purchaseorder po
/

/*
JSON_TABLE vs JSON_VALUE and JSON_QUERY

Select the requestor and the set of phones used by each object in 
column j_purchaseorder.po_document. 

First query parses that column four times, using JSON_TABLE, a single parse
*/
    
    
SELECT json_value(po_document, 
       '$.Requestor' RETURNING VARCHAR2(32)), 
       json_query(po_document, '$.ShippingInstructions.Phone'
          RETURNING VARCHAR2(100))
  FROM j_purchaseorder
 WHERE json_exists(po_document, '$.ShippingInstructions.Address.zipCode')
   AND json_value(po_document, '$.AllowPartialShipment' RETURNING VARCHAR2(5 CHAR)) = 'true'
/

/* or with JSON_TABLE */

SELECT jt.requestor, jt.phones FROM j_purchaseorder,
       json_table(po_document, '$'
          COLUMNS (requestor VARCHAR2(32 CHAR) PATH '$.Requestor',
                      phones VARCHAR2(100 CHAR) FORMAT JSON
                             PATH '$.ShippingInstructions.Phone',
                     partial VARCHAR2(5 CHAR) PATH '$.AllowPartialShipment'
                     has_zip VARCHAR2(5 CHAR) 
                        EXISTS PATH '$.ShippingInstructions.Address.zipCode')) jt
 WHERE jt.partial = 'true' AND jt.has_zip = 'true'
/

