ALTER SESSION SET plscope_settings='identifiers:all, statements:all'
/

DROP TABLE my_data
/

CREATE TABLE my_data (n NUMBER)
/

CREATE OR REPLACE PACKAGE my_pkg
   AUTHID DEFINER
IS
   FUNCTION my_function1
      RETURN my_data%ROWTYPE;

   FUNCTION my_function2
      RETURN my_data.n%TYPE;
END;
/

CREATE OR REPLACE FUNCTION my_function
   RETURN NUMBER
   AUTHID DEFINER
IS
BEGIN
   RETURN 1;
END;
/

CREATE OR REPLACE PROCEDURE my_procedure (n_in IN NUMBER)
   AUTHID DEFINER
IS
   l_n           my_data.n%TYPE;
   l_my_data     my_data%ROWTYPE;

   TYPE my_data_t IS TABLE OF my_data%ROWTYPE;

   l_array       my_data_t;
   l_returning   DBMS_SQL.number_table;
   l_cur         INTEGER := DBMS_SQL.open_cursor;
   l_cv          SYS_REFCURSOR;

   CURSOR all_data_cur
   IS
          SELECT *
            FROM my_data
      FOR UPDATE OF n;
BEGIN
   INSERT INTO my_data (n)
        VALUES (n_in);

   SELECT /*+ RESULT_CACHE */
          n
     INTO l_n
     FROM my_data
    WHERE n = n_in;

   SELECT *
     INTO l_my_data
     FROM my_data
    WHERE n = n_in;

   SELECT my_function ()
     INTO l_my_data
     FROM my_data
    WHERE n = n_in;

   SELECT COUNT (*)
     INTO l_my_data
     FROM my_data
    WHERE n = n_in;

   SELECT *
     BULK COLLECT INTO l_array
     FROM my_data;

       SELECT n
         INTO l_n
         FROM my_data
        WHERE n = n_in
   FOR UPDATE OF n;

   FOR rec IN all_data_cur
   LOOP
      UPDATE my_data
         SET n = rec.n * 10
       WHERE CURRENT OF all_data_cur;
   END LOOP;

   FORALL indx IN 1 .. l_array.COUNT
      DELETE FROM my_data
            WHERE n = l_array (indx).n
        RETURNING n
             BULK COLLECT INTO l_returning;

   EXECUTE IMMEDIATE 'UPDATE my_data SET n = n * :multiplier' USING 100;

   OPEN l_cv FOR 'SELECT * FROM my_data';

   DBMS_SQL.parse (l_cur,
                   'CREATE OR REPLACE TYPE my_type IS TABLE OF NUMBER',
                   DBMS_SQL.native);
END;
/

SELECT *
  FROM all_statements
 WHERE object_name = 'MY_PROCEDURE'
/

  SELECT TYPE,
         line,
         full_text,
         has_into_record,
         has_current_of,
         has_for_update,
         has_in_binds,
         has_into_bulk,
         usage_id,
         sql_id
    FROM all_statements
   WHERE object_name = 'MY_PROCEDURE'
ORDER BY usage_id
/

  SELECT name,
         TYPE,
         usage,
         usage_id,
         line
    FROM all_identifiers
   WHERE object_name = 'MY_PROCEDURE'
ORDER BY usage_id
/

/* All column usages in a program unit */

  SELECT idt.line,
         idt.owner || '.' || idt.object_name code_unit,
         idt.name column_name,
         RTRIM (src.text, CHR (10)) text
    FROM all_identifiers idt, all_source src
   WHERE     idt.usage = 'REFERENCE'
         AND idt.TYPE = 'COLUMN'
         AND idt.line = src.line
         AND idt.object_name = src.name
         AND idt.owner = src.owner
         AND idt.object_name = 'MY_PROCEDURE'
ORDER BY code_unit, line
/

/* Counts of DML operations across code base */

  SELECT owner, TYPE, COUNT (*)
    FROM all_statements
GROUP BY owner, object_name, TYPE
/

  SELECT owner,
         object_name,
         TYPE,
         COUNT (*)
    FROM all_statements
GROUP BY owner, object_name, TYPE
/

SELECT name,
       TYPE,
       usage,
       usage_id,
       usage_context_id,
       line
  FROM all_identifiers
 WHERE object_name = 'MY_PROCEDURE'
/

/* Find all program units with SQL containing hints */

SELECT owner,
       object_name,
       line,
       full_text
  FROM all_statements
 WHERE has_hint = 'YES'
/

/* Find all possible dynamic SQL injection points.
   Note: DBMS_SQL must be recompiled with PL/Scope turned on
   to get any results from the second half of the union all. */
SELECT st.owner,
       st.object_name,
       st.line,
       s.text
  FROM all_statements st, all_source s
 WHERE     st.TYPE IN ('EXECUTE IMMEDIATE', 'OPEN')
       AND st.owner = s.owner
       AND st.object_name = s.name
       AND st.line = s.line
UNION ALL
SELECT idnt.owner,
       idnt.object_name,
       idnt.line,
       src.text
  FROM all_identifiers idnt, all_source src
 WHERE     idnt.owner <> 'SYS'
       AND idnt.signature IN (SELECT a.signature
                                FROM all_identifiers a
                               WHERE     a.usage = 'DECLARATION'
                                     AND a.owner = 'SYS'
                                     AND a.object_name = 'DBMS_SQL'
                                     AND a.object_type = 'PACKAGE')
       AND idnt.owner = src.owner
       AND idnt.name = src.name
       AND idnt.line = src.line
/

/* Find all code units that reference the column identifier 
   table_in.column_in by first getting the identifier signature 
   and using that to find all REFERENCES to the identifier 
   that has that signature. */

CREATE OR REPLACE PROCEDURE show_column_usages (owner_in    IN VARCHAR2,
                                                table_in    IN VARCHAR2,
                                                column_in   IN VARCHAR2)
   AUTHID DEFINER
IS
BEGIN
   DBMS_OUTPUT.put_line (
      'References to ' || owner_in || '.' || table_in || '.' || column_in);
   DBMS_OUTPUT.put_line ('');

   FOR rec
      IN (  SELECT idt.line,
                   idt.owner || '.' || idt.object_name code_unit,
                   RTRIM (src.text, CHR (10)) text
              FROM all_identifiers idt, all_source src
             WHERE     idt.usage = 'REFERENCE'
                   AND idt.signature =
                          (SELECT idt_inner.signature
                             FROM all_identifiers idt_inner
                            WHERE     idt_inner.object_name =
                                         show_column_usages.table_in
                                  AND idt_inner.TYPE = 'COLUMN'
                                  AND idt_inner.name =
                                         show_column_usages.column_in
                                  AND idt_inner.usage = 'DECLARATION'
                                  AND idt_inner.owner = owner_in)
                   AND idt.line = src.line
                   AND idt.object_name = src.name
                   AND idt.owner = src.owner
          ORDER BY code_unit, line)
   LOOP
      DBMS_OUTPUT.put_line (
         'In ' || rec.code_unit || ' on ' || rec.line || ': ' || rec.text);
   END LOOP;
END;
/

/* Dynamic SQL occurrences */

SELECT st.owner,
       st.object_name,
       st.line,
       s.text
  FROM all_statements st, all_source s
 WHERE     st.TYPE IN ('EXECUTE IMMEDIATE', 'OPEN')
       AND st.owner = s.owner
       AND st.object_name = s.name
       AND st.line = s.line
/

/* Merge together the two views for a complete picture */

WITH one_obj_name AS (SELECT 'MY_PROCEDURE' object_name FROM DUAL)
    SELECT plscope_type,
           usage_id,
           usage_context_id,
           LPAD (' ', 2 * (LEVEL - 1)) || usage || ' ' || name usages,
           line,
           col,
           signature
      FROM (SELECT 'ID' plscope_type,
                   ai.object_name,
                   ai.usage usage,
                   ai.usage_id,
                   ai.usage_context_id,
                   ai.TYPE || ' ' || ai.name name,
                   ai.line,
                   ai.col,
                   signature
              FROM all_identifiers ai, one_obj_name
             WHERE ai.object_name = one_obj_name.object_name
            UNION ALL
            SELECT 'ST',
                   st.object_name,
                   st.TYPE,
                   st.usage_id,
                   st.usage_context_id,
                   'STATEMENT',
                   st.line,
                   st.col,
                   signature
              FROM all_statements st, one_obj_name
             WHERE st.object_name = one_obj_name.object_name)
START WITH usage_context_id = 0
CONNECT BY PRIOR usage_id = usage_context_id
/

/* All units that perform X operation on Y table */

CREATE OR REPLACE PROCEDURE show_dml_statements_on (
   owner_in            IN VARCHAR2,
   table_in            IN VARCHAR2,
   statement_type_in   IN VARCHAR2 DEFAULT 'ALL')
IS
BEGIN
   DBMS_OUTPUT.put_line (
         'Locations of '
      || statement_type_in
      || ' DML Statements Against '
      || owner_in
      || '.'
      || table_in);
   DBMS_OUTPUT.put_line ('');

   FOR rec
      IN (SELECT idt.line,
                 idt.owner || '.' || idt.object_name code_unit,
                 RTRIM (src.text, CHR (10)) text
            FROM all_identifiers idt, all_source src, all_statements st
           WHERE     idt.usage = 'REFERENCE'
                 AND idt.TYPE = 'TABLE'
                 AND idt.name = table_in
                 AND idt.owner = owner_in
                 AND idt.line = src.line
                 AND idt.object_name = src.name
                 AND idt.owner = src.owner
                 AND idt.usage_context_id = st.usage_id
                 AND (st.TYPE = statement_type_in OR statement_type_in = 'ALL'))
   LOOP
      DBMS_OUTPUT.put_line (
         'In ' || rec.code_unit || ' on ' || rec.line || ': ' || rec.text);
   END LOOP;
END;
/

BEGIN
   show_dml_statements_on (USER, 'ENDANGERED_SPECIES', 'INSERT');
END;
/