cl scr
set serveroutput on
/* Populate sparse array; use while to avoid NDF */
declare
  arr dbms_sql.number_table;
  inx pls_integer;
begin

  for i in 1 .. 10 loop
    if mod ( i, 2 ) = 0 then
      arr ( i ) := i;
    end if;
  end loop;
  
  inx := arr.first;
  while inx is not null loop
    dbms_output.put_line ( 'Val = ' || arr ( inx ) );
    inx := arr.next ( inx );
  end loop;

end;
/


/* Use extended iterator step to populate array 
   Indices of to walk through array */
declare
  arr dbms_sql.number_table;
begin

  for i in 2 .. 10 by 2 loop
    arr ( i ) := i;  
  end loop;
  
  for i in indices of arr loop
    dbms_output.put_line ( 'Val = ' || arr ( i ) );  
  end loop;

end;
/


/* Pairs to get array indices and values */
declare
  arr dbms_sql.number_table;
begin

  for i in 1 .. 5 loop
    arr ( i ) := i * 2;
  end loop;
  
  for inx, vals in pairs of arr loop
    dbms_output.put_line ( 
      'Index ' || inx || ' = ' || vals -- => arr ( inx )
    );
  end loop;

end;
/


/* Can specify fractional steps */
begin

  for i number /**/in 0 .. 1 by 0.2 loop
      dbms_output.put_line ( 'Val = ' || i );  
  end loop;

end;
/


/* Step value must be constant */
begin

  for i in 2 .. 100 by ( i * 2 ) loop
      dbms_output.put_line ( 'Val = ' || i );
  end loop;

end;
/



/* Use repeat .. while to provide a stepped expression */
begin

  for i in 1, repeat i * 2 while i < 100 loop
      dbms_output.put_line ( 'Val = ' || i );  
  end loop;

end;
/


/* Index variables are immutable by default
   Now you can declare them mutable to change these */
begin

  for i /*mutable /**/in 1 .. 100 loop
      dbms_output.put_line ( 'Val = ' || i );
      i := ( i * 2 ) - 1;
  end loop;

end;
/


/* Fibonacci numbers! */
declare
  prev pls_integer := 1;
  prev_prev pls_integer := 0;
begin

  dbms_output.put_line ( prev_prev );
  dbms_output.put_line ( prev );
  for fib mutable in 0 .. 100 loop

      fib := prev + prev_prev;
      dbms_output.put_line ( 'Val = ' || fib );
      
      prev_prev := prev;
      prev := fib;
        
  end loop;
  
end;
/







/* You can chain different iterators together! */
begin

  for i in reverse 1 .. 3, 
      repeat i * 2 while i <= 16, 
      10 .. 12 loop
      dbms_output.put_line ( 'Val = ' || i ); 
  end loop;

end;
/