create or replace package body p
is
   c_pkg constant char (1) := 'p';
   c_max_dopl_line integer := 255;
   c_delim constant char (3) := ' - ';
   /* Flag that determines whether output should be displayed. */
   v_show boolean := true;
   g_showtime boolean := false;
   g_lasttime integer := null;
   /* Private Variable for "linesep" */
   v_linesep varchar2 (100) := null;
   /* Private Variable for "prefix" */
   v_prefix varchar2 (100) := c_prefix;
   v_prefix_len integer := 1;
   v_linelen integer := c_linelen;

   /*------------------- Private Modules -------------------*/
   procedure put_line (stg_in in varchar2)
   is
   begin
      dbms_output.put_line (v_prefix || stg_in);
   exception
      when others
      then
         dbms_output.enable (1000000);
         dbms_output.put_line (v_prefix || stg_in);
   end;

   procedure display_line (show_in in boolean, line_in in varchar2)
   is
   begin
      if v_show or show_in
      then
         if rtrim (line_in) is null
         then
            put_line ('');
         elsif ltrim (rtrim (line_in)) = v_linesep
         then
            put_line ('');
         elsif length (line_in) > linelen
         then
            put_line (substr (line_in, 1, linelen));
            display_line (show_in
                        , substr (line_in, linelen + 1));
         else
            put_line (line_in);
         end if;
      end if;
   end;

   /*--------------- Toggle Output Modules ---------------*/
   procedure turn_on
   is
   begin
      v_show := true;
      dbms_output.enable (1000000);
   end;

   procedure turn_off
   is
   begin
      v_show := false;
   end;

   /* Body of Set for "linesep" */
   procedure set_linesep (linesep_in in varchar2)
   is
   begin
      v_linesep := linesep_in;
   end set_linesep;

   /* Body of Get for "linesep" */
   function linesep
      return varchar2
   is
   begin
      return v_linesep;
   end;

   /* Body of Set for "prefix" */
   procedure set_prefix (prefix_in in varchar2 := c_prefix)
   is
   begin
      v_prefix := prefix_in;
      v_prefix_len := length (prefix_in);
   end set_prefix;

   /* Body of Get for "prefix" */
   function prefix
      return varchar2
   is
   begin
      return v_prefix;
   end prefix;

   /* Set line length before wrap */
   procedure set_linelen (len in integer := c_linelen)
   is
   begin
      v_linelen :=
         least (c_max_dopl_line
              , greatest (nvl (len, c_linelen), 1)
               );
   end;

   function linelen
      return integer
   is
   begin
      return v_linelen;
   end;

   /*------------------ The p.l Procedures ----------------*/
   function date_string (dt in date, mask in varchar2 := null)
      return varchar2
   is
   begin
      if mask is not null
      then
         return to_char (dt, mask);
      else
         return to_char (dt);
      end if;
   end;

   function bool_string (val in boolean)
      return varchar2
   is
   begin
      if val
      then
         return 'TRUE';
      elsif not val
      then
         return 'FALSE';
      else
         return 'NULL';
      end if;
   end;

   procedure l (
      dt in date
    , mask_in in varchar2 := null
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in, date_string (dt, mask_in));
   end;

   procedure l (num in number, show_in in boolean := false)
   is
   begin
      display_line (show_in, to_char (num));
   end;

   procedure l (stg in varchar2, show_in in boolean := false)
   is
   begin
      display_line (show_in, stg);
   end;

   procedure l (
      stg in varchar2
    , num in number
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in, stg || c_delim || to_char (num));
   end;

   procedure l (
      stg in varchar2
    , dt in date
    , mask_in in varchar2 := null
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in
                  , stg || c_delim || date_string (dt, mask_in)
                   );
   end;

   procedure l (bool in boolean, show_in in boolean := false)
   is
   begin
      display_line (show_in, bool_string (bool));
   end;

   procedure l (
      stg in varchar2
    , bool in boolean
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in
                  , stg || c_delim || bool_string (bool)
                   );
   end;

   procedure l (
      file_in in utl_file.file_type
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in, to_char (file_in.id));
   end;

   procedure l (
      string_in in varchar2
    , file_in in utl_file.file_type
    , show_in in boolean := false
   )
   is
   begin
      l (string_in, file_in.id, show_in);
   end;

   /* Additional overloadings */
   procedure l (
      num1 in number
    , num2 in number
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in
                  , to_char (num1) || c_delim || to_char (num2)
                   );
   end;

   procedure l (
      str in varchar2
    , num1 in number
    , num2 in number
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in
                  ,    str
                    || c_delim
                    || to_char (num1)
                    || c_delim
                    || to_char (num2)
                   );
   end;

   procedure l (
      bool1 in boolean
    , bool2 in boolean
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in
                  ,    bool_string (bool1)
                    || c_delim
                    || bool_string (bool2)
                   );
   end;

   procedure l (
      stg1 in varchar2
    , stg2 in varchar2
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in, stg1 || c_delim || stg2);
   end;

   procedure l (
      dt1 in date
    , dt2 in date
    , mask_in in varchar2 := null
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in
                  ,    date_string (dt1, mask_in)
                    || c_delim
                    || date_string (dt2, mask_in)
                   );
   end;

   procedure l (
      num in number
    , dt in date
    , mask_in in varchar2 := null
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in
                  ,    to_char (num)
                    || c_delim
                    || date_string (dt, mask_in)
                   );
   end;

   procedure l (
      bool in boolean
    , num in number
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in
                  , bool_string (bool) || c_delim
                    || to_char (num)
                   );
   end;

   procedure l (
      bool in boolean
    , dt in date
    , mask_in in varchar2 := null
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in
                  ,    bool_string (bool)
                    || c_delim
                    || date_string (dt, mask_in)
                   );
   end;

   procedure l (
      bool in boolean
    , stg in varchar2
    , show_in in boolean := false
   )
   is
   begin
      display_line (show_in
                  , stg || c_delim || bool_string (bool)
                   );
   end;

   /* Show time stamp toggle. Default is NOSHOW. */
   procedure showtime
   is
   begin
      g_showtime := true;
      g_lasttime := null;
   end;

   procedure noshowtime
   is
   begin
      g_showtime := true;
      g_lasttime := null;
   end;

   function showing_time
      return boolean
   is
   begin
      return g_showtime;
   end;
end p;
/
