DECLARE
   l_index integer := 1;
   l_actions   alert_mgr.actions_t;
   c_body constant varchar2(4000):= '<html>
<head>
<title>Big Earthquake Alert</title>
</head>
<body>
<h1>Big Earthquake Alert!</h1>
<p>Check out all 6.5+ earthquakes at 
<a html="http://apexcentral.us.oracle.com:7779/apx201dev/mike/r/earthquakes3/">
   Earthquake Central</a></p>
</body>
</html>';
BEGIN
   DELETE FROM earthquake_alerts;
   
   /* Single email. */
   l_index := 1;
   l_actions (l_index).action_type := 'EMAIL';
   l_actions (l_index).send_email_to := 'steven.feuerstein@oracle.com,michael.hichwa@oracle.com,david.peake@oracle.com';
   l_actions (l_index).email_subject := 'Earthquake > 6.5 Reported!';
   l_actions (l_index).email_body := c_body;

   /* Multiple emails */
   l_index := 2;
   l_actions (l_index).action_type := 'EMAIL';
   l_actions (l_index).send_email_to := 'steven.feuerstein@oracle.com,michael.hichwa@oracle.com,david.peake@oracle.com';
   l_actions (l_index).email_query := q'[
   SELECT 'Magnitude ' || magnitude || ' Earthquake detected in ' || location subject,
          'Earthquake ID ' || id body
     FROM earthquakes 
    WHERE magnitude >= 6.5
      AND id NOT IN (SELECT earthquake_id FROM earthquake_alerts)]';
   
   /* Record as notified */
   l_index := 3;
   l_actions (l_index).action_type := 'PROC';
   l_actions (l_index).action_code := 'earthquake_mgr.record_notifications;';
   
   /* Record in log */
   l_index := 4;
   l_actions (l_index).action_type := 'PROC';
   l_actions (l_index).action_code := 
      'alert_mgr.log(''Earthquakes checked at '' || TO_CHAR (SYSDATE, ''YYYY-MM-DD HH24:MI:SS''))';
      
   alert_mgr.register_alert (
      alert_name_in             => 'Notify at 6.5',
      description_in            =>    'Send an email whenever an earthquake rated at 6.5 or higher occurs. '
                                   || 'Do not send a notification twice for the same earthquake.',
      trigger_code_in           => 'SELECT 1 FROM earthquakes WHERE magnitude >= 6.5
   AND id NOT IN (SELECT earthquake_id FROM earthquake_alerts)',
      trigger_type_in           => alert_mgr.c_trigger_type_select_mult_rows,
      check_frequency_in        => '1',
      recurrence_frequency_in   => '1',
      ignore_failures_in        => 'Y',
      actions_in => l_actions,
      is_active_in              => 'Y');  
END;
/