CREATE OR REPLACE FUNCTION exprep_get_approvers (exprep_id IN NUMBER)
   RETURN VARCHAR2
IS
   l_return   employees.name%TYPE;
BEGIN
   SELECT CASE
             WHEN er.amount < 100 THEN e.name
             WHEN er.amount BETWEEN 100 AND 1000 THEN m.name
             WHEN er.amount > 1000 THEN 
             case  when status = 'SUBMITTED' then m.name else mm.name end
          END
     INTO l_return
     FROM expense_report er,
          employees e,
          employees m,
          employees mm
    WHERE     er.id = exprep_id
          AND er.submitter_id = e.id
          AND e.manager_id = m.id
          AND m.manager_id = mm.id(+);

   RETURN l_return;
END;
/

CREATE OR REPLACE PROCEDURE approve_expense_report (exprep_id IN NUMBER)
IS
BEGIN
   UPDATE expense_report
      SET status = 'APPROVED'
    WHERE id = exprep_id;
END;
/

DECLARE
   l_actions   alert_mgr.actions_t;
BEGIN
   l_actions (1).action_type := 'EMAIL';
   l_actions (1).email_subject := 'Expense report #KEY_VALUE# approved';
   l_actions (1).email_body :=
      '<p>Payment for expense report named "#KEY_NAME# will be expedited.</p>';
   l_actions (2).action_type := 'PROC';
   l_actions (2).action_code := 'approve_expense_report (#KEY_VALUE#)';

   alert_mgr.register_alert (
      alert_name_in               => 'Auto Approve Low Total Expense Reports',
      description_in              => 'Auto approve expense reports < $100 and notify user via email with that expense report info.',
      trigger_code_in             => q'[SELECT id FROM expense_reports
 WHERE amount < 100 AND status = 'SUBMITTED']',
      trigger_type_in             => 'MULTROWS',
      check_frequency_in          => '5',
      recurrence_frequency_in     => '5',
      ignore_failures_in          => 'Y',
      actions_in                  => l_actions,
      is_active_in                => 'Y',
      trigger_key_query_in        => 'HOW WOULD I USE THIS?',
      target_identifer_code_in    => 'exprep_get_emails (#KEY_VALUE#)',
      repeat_for_trigger_key_in   => 'N',
      repeat_for_target_key_in    => 'N');
END;
/

DECLARE
   l_actions   alert_mgr.actions_t;
BEGIN
   l_actions (1).action_type := 'EMAIL';
   l_actions (1).email_subject := 'Expense report #KEY_VALUE# must be approved';
   l_actions (1).email_body :=
      '<p>Payment for expense report named "#KEY_NAME# will be expedited.</p>';

   alert_mgr.register_alert (
      alert_name_in               => 'Manager Approves Expense Reports',
      description_in              => 'Manager must approve expense reports with totals between 100 and 1000.',
      trigger_code_in             => q'[SELECT id FROM expense_reports
 WHERE amount BETWEEN 100 AND 1000 AND status = 'SUBMITTED']',
      trigger_type_in             => 'MULTROWS',
      check_frequency_in          => '5',
      recurrence_frequency_in     => '5',
      ignore_failures_in          => 'Y',
      actions_in                  => l_actions,
      is_active_in                => 'Y',
      trigger_key_query_in        => 'HOW WOULD I USE THIS?',
      target_identifer_code_in    => 'exprep_get_emails (#KEY_VALUE#)',
      repeat_for_trigger_key_in   => 'N',
      repeat_for_target_key_in    => 'N');
END;
/

DECLARE
   l_actions   alert_mgr.actions_t;
BEGIN
   l_actions (1).action_type := 'EMAIL';
   l_actions (1).email_subject := 'Expense report #KEY_VALUE# approved';
   l_actions (1).email_body :=
      '<p>Payment for expense report named "#KEY_NAME# will be expedited.</p>';

   alert_mgr.register_alert (
      alert_name_in               => 'Manager''s Manager Approves BIG Expense Reports',
      description_in              => 'Manager's manager must approve expense reports with totals > 1000.',
      trigger_code_in             => q'[SELECT id FROM expense_reports
 WHERE amount > 1000 AND status = 'SUBMITTED']',
      trigger_type_in             => 'MULTROWS',
      check_frequency_in          => '5',
      recurrence_frequency_in     => '5',
      ignore_failures_in          => 'Y',
      actions_in                  => l_actions,
      is_active_in                => 'Y',
      trigger_key_query_in        => 'HOW WOULD I USE THIS?',
      target_identifer_code_in    => 'exprep_get_emails (#KEY_VALUE#)',
      repeat_for_trigger_key_in   => 'N',
      repeat_for_target_key_in    => 'N');
END;
/