/*
Demonstrates how to declare an associative array type, populate that array non-sequentially,
and iterate through the elements using methods.
*/

DECLARE
   TYPE list_of_names_t IS TABLE OF VARCHAR2 (32767)
      INDEX BY PLS_INTEGER;

   emp_names       list_of_names_t;
   l_index_value   PLS_INTEGER;
BEGIN
   SELECT last_name
     BULK COLLECT INTO emp_names
     FROM hr.employees;

   FOR indx IN 1 .. emp_names.COUNT
   LOOP
      DBMS_OUTPUT.put_line (
         'Name at index ' || indx || ' = ' || emp_names (indx));
   END LOOP;

   emp_names.delete;

   emp_names (100) := 'Ellison';
   emp_names (-15070) := 'Catz';

   DBMS_OUTPUT.put_line (emp_names.COUNT);

   l_index_value := emp_names.FIRST;

   WHILE (l_index_value IS NOT NULL)
   LOOP
      DBMS_OUTPUT.put_line (
            'Name at index '
         || l_index_value
         || ' = '
         || emp_names (l_index_value));

      l_index_value := happyfamily.NEXT (l_index_value);
   END LOOP;
END;
/