set timing on

drop TABLE error_log 
/

CREATE TABLE error_log 
( 
   log_id       NUMBER GENERATED ALWAYS AS IDENTITY, 
   created_on   TIMESTAMP WITH LOCAL TIME ZONE, 
   created_by   VARCHAR2 (100), 
   errorcode    INTEGER, 
   callstack    VARCHAR2 (4000), 
   errorstack   VARCHAR2 (4000), 
   backtrace    VARCHAR2 (4000), 
   error_info   VARCHAR2 (4000) 
)
/

create or replace procedure log_error_cs4 (
   app_info_in in varchar2
) is
   pragma AUTONOMOUS_TRANSACTION;
   c_code constant integer := SQLCODE;
begin
   insert into error_log (
      created_on,
      created_by,
      errorcode,
      callstack,
      errorstack,
      backtrace,
      error_info
   ) values (
      SYSTIMESTAMP,
      USER,
      c_code,
      DBMS_UTILITY.format_call_stack,
      DBMS_UTILITY.format_error_stack,
      DBMS_UTILITY.format_error_backtrace,
      app_info_in
   );

   commit;
end;
/

create or replace procedure log_error_cs1 (
   app_info_in in varchar2
) is
   pragma AUTONOMOUS_TRANSACTION;
   c_code    constant integer := SQLCODE;
   c_cstack  constant varchar2(32767) := DBMS_UTILITY.format_call_stack;
   c_estack  constant varchar2(32767) := DBMS_UTILITY.format_error_stack;
   c_etrace  constant varchar2(32767) := DBMS_UTILITY.format_error_backtrace;
begin
   insert into error_log (
      created_on,
      created_by,
      errorcode,
      callstack,
      errorstack,
      backtrace,
      error_info
   ) values (
      SYSTIMESTAMP,
      USER,
      c_code,
      c_cstack,
      c_estack,
      c_etrace,
      app_info_in
   );

   commit;
end;
/

begin
   for indx in 1..100000 loop
      begin
         raise no_data_found;
      exception
         when others then
            log_error_cs4(indx);
      end;
   end loop;
end;
/

begin
   for indx in 1..100000 loop
      begin
         raise no_data_found;
      exception
         when others then
            log_error_cs1(indx);
      end;
   end loop;
end;
/