SET SERVEROUTPUT ON FORMAT WRAPPED

CREATE OR REPLACE FUNCTION plch_betwnstr (
   string_in   IN   VARCHAR2
 , start_in    IN   INTEGER
 , end_in      IN   INTEGER
)
   RETURN VARCHAR2 AUTHID DEFINER
IS
BEGIN
   DBMS_LOCK.sleep (.025);
   DBMS_OUTPUT.put_line ('Called betwnstr');
   RETURN (SUBSTR (string_in, start_in, end_in - start_in + 1));
END;
/

DECLARE
   l_start   PLS_INTEGER;
BEGIN
   l_start := DBMS_UTILITY.get_time;

   FOR rec IN (SELECT plch_betwnstr ('FEUERSTEIN', 1, 5)
                 FROM employees
                WHERE ROWNUM < 101)
   LOOP
      NULL;
   END LOOP;

   DBMS_OUTPUT.put_line (   'Non-deterministic in query elapsed: '
                         || TO_CHAR (DBMS_UTILITY.get_time - l_start)
                        );
END;
/

CREATE OR REPLACE FUNCTION plch_betwnstr (
   string_in   IN   VARCHAR2
 , start_in    IN   INTEGER
 , end_in      IN   INTEGER
)
   RETURN VARCHAR2 DETERMINISTIC AUTHID DEFINER
IS
BEGIN
   DBMS_LOCK.sleep (.025);
   DBMS_OUTPUT.put_line ('Called betwnstr');
   RETURN (SUBSTR (string_in, start_in, end_in - start_in + 1));
END;
/

DECLARE
   l_start   PLS_INTEGER;
BEGIN
   l_start := DBMS_UTILITY.get_time;

   FOR rec IN (SELECT plch_betwnstr ('FEUERSTEIN', 1, 5)
                 FROM employees
                WHERE ROWNUM < 101)
   LOOP
      NULL;
   END LOOP;

   DBMS_OUTPUT.put_line (   'Deterministic in query lapsed: '
                         || TO_CHAR (DBMS_UTILITY.get_time - l_start)
                        );
END;
/

DECLARE
   l_string   VARCHAR2 (100);
   l_start    PLS_INTEGER;
BEGIN
   l_start := DBMS_UTILITY.get_time;

   FOR rec IN (SELECT *
                 FROM employees
                WHERE ROWNUM < 101)
   LOOP
      l_string := plch_betwnstr ('FEUERSTEIN', 1, 5);
   END LOOP;

   DBMS_OUTPUT.put_line (   'Deterministic in block elapsed: '
                         || TO_CHAR (DBMS_UTILITY.get_time - l_start)
                        );
END;
/

/* Now move plch_betwnstr into a SELECT FROM DUAL -
   NO BENEFIT...so the caching is only lasting for as long as the query!
*/
DECLARE
   l_string   VARCHAR2 (100);
   l_start    PLS_INTEGER;
BEGIN
   l_start := DBMS_UTILITY.get_time;

   FOR rec IN (SELECT *
                 FROM employees
                WHERE ROWNUM < 101)
   LOOP
      SELECT plch_betwnstr ('FEUERSTEIN', 1, 5)
        INTO l_string
        FROM DUAL;
   END LOOP;

   DBMS_OUTPUT.put_line (   'Deterministic in SELECT from DUAL elapsed: '
                         || TO_CHAR (DBMS_UTILITY.get_time - l_start)
                        );
END;
/

DROP FUNCTION plch_betwnstr
/