CREATE OR REPLACE PROCEDURE drop_dangling_bodies (
   nm             IN VARCHAR2 DEFAULT '%',
   test_only_in   IN BOOLEAN DEFAULT TRUE)
   AUTHID CURRENT_USER
IS
   PRAGMA AUTONOMOUS_TRANSACTION;
   dropstr   VARCHAR2 (2000);

   CURSOR object_cur (spec_type_in   IN VARCHAR2,
                      body_type_in   IN VARCHAR2)
   IS
      SELECT object_name
        FROM user_objects
       WHERE object_type = body_type_in AND object_name LIKE nm
      MINUS
      SELECT object_name
        FROM user_objects
       WHERE object_type = spec_type_in AND object_name LIKE nm;

   PROCEDURE drop_type (spec_type_in   IN VARCHAR2,
                        body_type_in   IN VARCHAR2)
   IS
   BEGIN
      FOR rec IN object_cur (spec_type_in, body_type_in)
      LOOP
         dropstr :=
            'DROP ' || body_type_in || ' ' || rec.object_name;

         IF test_only_in
         THEN
            DBMS_OUTPUT.put_line (
               dropstr || ' - VERIFICATION ONLY - NOT DROPPED!');
         ELSE
            BEGIN
               EXECUTE IMMEDIATE dropstr;

               DBMS_OUTPUT.put_line (
                  dropstr || ' - SUCCESSFUL!');
            EXCEPTION
               WHEN OTHERS
               THEN
                  DBMS_OUTPUT.put_line (
                     dropstr || ' - FAILURE!');
                  DBMS_OUTPUT.put_line (
                     DBMS_UTILITY.FORMAT_ERROR_STACK);
            END;
         END IF;
      END LOOP;
   END drop_type;
BEGIN
   drop_type ('PACKAGE', 'PACKAGE BODY');
   drop_type ('TYPE', 'TYPE BODY');
END drop_dangling_bodies;
/