create or replace procedure gen_pkg_encap (package_in in varchar2, schema_in in varchar2 default user, spec_out out clob, body_out out clob)
is
	   SUBTYPE parameter_position_t is PLS_INTEGER;
   SUBTYPE overloading_#_t IS PLS_INTEGER;
   SUBTYPE subprogram_name_t IS all_arguments.object_name%TYPE;

      c_top_level        CONSTANT PLS_INTEGER := 0;
   c_return_pos       CONSTANT PLS_INTEGER := 0;
   --
   -- Parameter mode values used in DBMS_DESCRIBE
   c_in               CONSTANT PLS_INTEGER := 0;
   c_out              CONSTANT PLS_INTEGER := 1;
   c_inout            CONSTANT PLS_INTEGER := 2;
   --
   -- Indicators for presence of default value
   c_has_default      CONSTANT PLS_INTEGER := 1;
   c_has_no_default   CONSTANT PLS_INTEGER := 0;

   /* A single record holding all the information for one argument. */
   TYPE one_argument_rt IS RECORD
   (
      owner                all_arguments.owner%TYPE,
      package_name         all_arguments.package_name%TYPE,
      object_name          all_arguments.object_name%TYPE,
      overload             PLS_INTEGER,                  -- from DBMS_DESCRIBE
      POSITION             all_arguments.position%TYPE,
      data_level           all_arguments.data_level%TYPE,
      argument_name        all_arguments.argument_name%TYPE,
      datatype             PLS_INTEGER,                  -- from DBMS_DESCRIBE
      DEFAULT_VALUE        PLS_INTEGER,                  -- from DBMS_DESCRIBE
      in_out               PLS_INTEGER,                  -- from DBMS_DESCRIBE
      character_set_name   all_arguments.character_set_name%TYPE,
      type_owner           all_arguments.type_owner%TYPE,
      type_name            all_arguments.type_name%TYPE,
      type_subname         all_arguments.type_subname%TYPE,
      type_link            all_arguments.type_link%TYPE,
      pls_type             all_arguments.pls_type%TYPE,
      char_length          all_arguments.char_length%TYPE,
      char_used            all_arguments.char_used%TYPE
   );

      -- Contents of ALL_ARGS/DBMS_DESCRIBE, loaded sequentially
   TYPE arguments_tt IS TABLE OF one_argument_rt
      INDEX BY BINARY_INTEGER;
   
   -- List of top level parameters (level = 0 in ALL_ARGS)
   TYPE parameters_tt IS TABLE OF cc_arguments.arguments_tt
      INDEX BY parameter_position_t;

   -- Overloadings for a single program name
   TYPE one_overloading_rt IS RECORD (
      PARAMETERS             parameters_tt
     ,return_clause          one_parameter_rt
   );

   TYPE overloadings_t IS TABLE OF one_overloading_rt
      INDEX BY overloading_#_t;

   TYPE programs_t IS TABLE OF overloadings_t
      INDEX BY subprogram_name_t;

      FUNCTION fullset (program_in IN VARCHAR2, show_in IN BOOLEAN := FALSE)
      RETURN arguments_tt
   IS
      l_index           PLS_INTEGER;
      l_names           cc_names.names_rt;
      retval            arguments_tt;

      CURSOR distinct_programs_cur (
         owner_in          IN VARCHAR2,
         package_name_in   IN VARCHAR2,
         object_name_in    IN VARCHAR2)
      IS
         SELECT DISTINCT owner, package_name, object_name
           FROM all_arguments
          WHERE     owner = owner_in
                AND (   (    package_name = package_name_in
                         AND (   object_name = object_name_in
                              OR object_name_in IS NULL))
                     OR (    package_name IS NULL
                         AND package_name_in IS NULL
                         AND object_name = object_name_in));

      /* PL/SQL tables which will hold the output from calls to
      DBMS_DESCRIBE.DESCRIBE_PROCEDURE */
      l_overload        DBMS_DESCRIBE.number_table;
      l_position        DBMS_DESCRIBE.number_table;
      l_level           DBMS_DESCRIBE.number_table;
      l_argument_name   DBMS_DESCRIBE.varchar2_table;
      l_datatype        DBMS_DESCRIBE.number_table;
      l_default_value   DBMS_DESCRIBE.number_table;
      l_in_out          DBMS_DESCRIBE.number_table;
      l_length          DBMS_DESCRIBE.number_table;
      l_precision       DBMS_DESCRIBE.number_table;
      l_scale           DBMS_DESCRIBE.number_table;
      l_radix           DBMS_DESCRIBE.number_table;
      l_spare           DBMS_DESCRIBE.number_table;

      PROCEDURE get_dbms_describe_info (
         rec_in   IN distinct_programs_cur%ROWTYPE)
      IS
         l_name   cc_names.combined_name_t
            := cc_names.combined (rec_in.owner,
                                  rec_in.package_name,
                                  rec_in.object_name);
      BEGIN
         DBMS_DESCRIBE.describe_procedure (l_name,
                                           NULL,
                                           NULL,
                                           l_overload,
                                           l_position,
                                           l_level,
                                           l_argument_name,
                                           l_datatype,
                                           l_default_value,
                                           l_in_out,
                                           l_length,
                                           l_precision,
                                           l_scale,
                                           l_radix,
                                           l_spare);
      END;

      FUNCTION onerow (object_name_in     IN VARCHAR2,
                    package_name_in    IN VARCHAR2,
                    owner_in           IN VARCHAR2,
                    overload_in        IN PLS_INTEGER,
                    argument_name_in   IN VARCHAR2,
                    position_in        IN PLS_INTEGER,
                    level_in           IN PLS_INTEGER)
      RETURN all_arguments%ROWTYPE
   IS
      empty_rec   all_arguments%ROWTYPE;
      retval      all_arguments%ROWTYPE;
   BEGIN
      SELECT *
        INTO retval
        FROM all_arguments
       WHERE     object_name = object_name_in
             AND (   package_name = package_name_in
                  OR (package_name IS NULL AND package_name_in IS NULL))
             AND owner = owner_in
             AND (   overload = overload_in
                  OR (    overload IS NULL
                      AND (overload_in = 0 OR overload_in IS NULL)))
             AND POSITION = position_in
             AND LEVEL = level_in
             AND argument_name = argument_name_in;

      RETURN retval;
   EXCEPTION
      WHEN NO_DATA_FOUND
      THEN
         RETURN empty_rec;
   END;

   FUNCTION for_program (program_in IN VARCHAR2)
      RETURN names_rt
   IS
      TYPE notused_rt IS RECORD (
         must_be_one     PLS_INTEGER
        ,dblink          VARCHAR2 (100)
        ,object_number   NUMBER
      );

      l_notused   notused_rt;
      retval      names_rt;
      empty       names_rt;
   BEGIN
      l_notused.must_be_one := 1;
      DBMS_UTILITY.name_resolve (program_in
                                ,l_notused.must_be_one
                                ,retval.owner
                                ,retval.package_name
                                ,retval.object_name
                                ,l_notused.dblink
                                ,retval.object_type
                                ,l_notused.object_number
                                );
      retval.ispackage := is_package (retval.object_type);
      RETURN retval;
   EXCEPTION
      WHEN OTHERS
      THEN
         RETURN empty;
   END for_program;

      PROCEDURE transfer_data (rec    IN distinct_programs_cur%ROWTYPE,
                               indx   IN PLS_INTEGER)
      IS
         l_argindx   PLS_INTEGER := NVL (retval.LAST, 0) + 1;

         PROCEDURE copy_to_arguments_array (
            rec_in       IN distinct_programs_cur%ROWTYPE,
            ddindx_in    IN PLS_INTEGER,
            argindx_in   IN PLS_INTEGER)
         IS
         BEGIN
            retval (argindx_in).object_name := rec_in.object_name;
            retval (argindx_in).package_name := rec_in.package_name;
            retval (argindx_in).owner := rec_in.owner;
            retval (argindx_in).overload := l_overload (ddindx_in);
            retval (argindx_in).POSITION := l_position (ddindx_in);
            retval (argindx_in).data_level := l_level (ddindx_in);
            retval (argindx_in).argument_name := l_argument_name (ddindx_in);
            retval (argindx_in).datatype := l_datatype (ddindx_in);
            retval (argindx_in).DEFAULT_VALUE := l_default_value (ddindx_in);
            retval (argindx_in).in_out := l_in_out (ddindx_in);
         END;

         PROCEDURE add_all_arguments_info (
            argument_inout   IN OUT one_argument_rt)
         IS
            l_argument   all_arguments%ROWTYPE
                            := onerow (argument_inout.object_name,
                                       argument_inout.package_name,
                                       argument_inout.owner,
                                       argument_inout.overload,
                                       argument_inout.argument_name,
                                       argument_inout.POSITION,
                                       argument_inout.data_level);
         BEGIN
            argument_inout.character_set_name := l_argument.character_set_name;
            argument_inout.type_owner := l_argument.type_owner;
            argument_inout.type_name := l_argument.type_name;
            argument_inout.type_subname := l_argument.type_subname;
            argument_inout.type_link := l_argument.type_link;
            argument_inout.pls_type := l_argument.pls_type;
            argument_inout.char_length := l_argument.char_length;
            argument_inout.char_used := l_argument.char_used;
         END add_all_arguments_info;
      BEGIN
         copy_to_arguments_array (rec, indx, l_argindx);
         add_all_arguments_info (retval (l_argindx));
      END transfer_data;
   BEGIN                                                      
      l_names := for_program (package_in);

      FOR rec
         IN distinct_programs_cur (l_names.owner,
                                   l_names.package_name,
                                   l_names.object_name)
      LOOP
         get_dbms_describe_info (rec);
         l_index := l_argument_name.FIRST;

         LOOP
            EXIT WHEN l_index IS NULL;
            transfer_data (rec, l_index);
            l_index := l_argument_name.NEXT (l_index);
         END LOOP;
      END LOOP;

      RETURN retval;
   END fullset;
begin
  
select * bulk collect into l_arguments from user_arguments where owner = schema_in and package_name = package_in;
select distinct object_name bulk collect into
l_subprograms  from user_arguments where owner = schema_in and package_name = package_in;

for indx in (select)
end;