SET SERVEROUTPUT ON


CREATE TABLE qz_data
(
   n   NUMBER (3),
   s   VARCHAR2 (9),
   c   DATE
);

BEGIN
   INSERT INTO qz_data (n, s)
        VALUES (100, 'Sam');

   INSERT INTO qz_data (n, s)
        VALUES (101, 'Bill');

   INSERT INTO qz_data (n, s)
        VALUES (102, 'Juna');

   INSERT INTO qz_data (n, s)
        VALUES (103, 'Harry');

   INSERT INTO qz_data (n, s)
        VALUES (104, 'Lisa');

   INSERT INTO qz_data (n, s)
        VALUES (105, 'James');

   INSERT INTO qz_data (n, s)
        VALUES (106, 'Lakshmi');

   INSERT INTO qz_data (n, s)
        VALUES (107, 'Manuel');

   INSERT INTO qz_data (n, s)
        VALUES (108, 'Sally');

   INSERT INTO qz_data (n, s)
        VALUES (109, 'Sharon');

   COMMIT;
END;
/

BEGIN
   DBMS_ERRLOG.create_error_log (dml_table_name => 'QZ_DATA');
END;
/

CREATE OR REPLACE PACKAGE qz_pkg
IS
   TYPE ids_t IS TABLE OF INTEGER;
END;
/

CREATE OR REPLACE TRIGGER qz_data_tr
   BEFORE INSERT OR UPDATE
   ON qz_data
   FOR EACH ROW
DECLARE
BEGIN
   :new.c := SYSDATE;
END;
/

/*
Attempt 6 row updates in batches of 2 - only one row update should succeed and 
five should fail, but log errors limit of 3 is applied cumulatively with FORALL 
*/

DECLARE
   l_lows    qz_pkg.ids_t := qz_pkg.ids_t (102, 106, 108);
   l_highs   qz_pkg.ids_t := qz_pkg.ids_t (103, 107, 109);
   l_count   INTEGER;
BEGIN
   FORALL indx IN 1 .. l_lows.COUNT
      UPDATE qz_data
         SET s = s || s
       WHERE n BETWEEN l_lows (indx) AND l_highs (indx)
       LOG ERRORS REJECT LIMIT 3;

   SELECT COUNT (*)
     INTO l_count
     FROM qz_data
    WHERE c IS NOT NULL;

   DBMS_OUTPUT.put_line (
      'FORALL : No Exception : Successfully updated rows = ' || l_count);

   SELECT COUNT (*) INTO l_count FROM err$_qz_data;

   DBMS_OUTPUT.put_line ('FORALL : No Exception : Failed rows = ' || l_count);
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.put_line (
         'FORALL : Failed with : ' || DBMS_UTILITY.format_error_stack);

      SELECT COUNT (*) INTO l_count FROM err$_qz_data;

      DBMS_OUTPUT.put_line ('FORALL : Exception : Failed rows = ' || l_count);
END;
/

ROLLBACK
/

TRUNCATE TABLE err$_qz_data
/

/*
Attempt 6 row updates in batches of 2 - only one row update should succeed and 
five should fail, but log errors limit of 3 is applied individually with FOR 
*/

DECLARE
   l_lows    qz_pkg.ids_t := qz_pkg.ids_t (102, 106, 108);
   l_highs   qz_pkg.ids_t := qz_pkg.ids_t (103, 107, 109);
   l_count   INTEGER;
BEGIN
   FOR indx IN 1 .. l_lows.COUNT
   LOOP
      UPDATE qz_data
         SET s = s || s
       WHERE n BETWEEN l_lows (indx) AND l_highs (indx)
       LOG ERRORS REJECT LIMIT 3;
   END LOOP;

   SELECT COUNT (*)
     INTO l_count
     FROM qz_data
    WHERE c IS NOT NULL;

   DBMS_OUTPUT.put_line (
      'FOR : No Exception : Successfully updated rows = ' || l_count);

   SELECT COUNT (*) INTO l_count FROM err$_qz_data;

   DBMS_OUTPUT.put_line ('FOR : No Exception : Failed rows = ' || l_count);
EXCEPTION
   WHEN OTHERS
   THEN
      DBMS_OUTPUT.put_line (
         'FOR : Failed with : ' || DBMS_UTILITY.format_error_stack);

      SELECT COUNT (*) INTO l_count FROM err$_qz_data;

      DBMS_OUTPUT.put_line ('FOR : Exception : Failed rows = ' || l_count);
END;
/

DROP TABLE qz_data
/

DROP TABLE err$_qz_data
/

DROP PACKAGE qz_pkg
/