DROP TABLE qz_limbs
/

CREATE TABLE qz_limbs
(
   nm        VARCHAR2 (100),
   avg_len   NUMBER
)
/

BEGIN
   INSERT INTO qz_limbs
        VALUES (1, 'arm');

   INSERT INTO qz_limbs
        VALUES (2, 'leg');

   INSERT INTO qz_limbs
        VALUES (3, 'tail');

   COMMIT;
END;
/

CREATE OR REPLACE TYPE qz_limb_ot IS OBJECT
(
   nm VARCHAR2 (100),
   avg_len NUMBER,
   MAP MEMBER FUNCTION limb_map
      RETURN NUMBER
)
/

CREATE OR REPLACE TYPE BODY qz_limb_ot
IS
   MAP MEMBER FUNCTION food_mapping
      RETURN NUMBER
   IS
   BEGIN
      RETURN LENGTH (self.nm);
   END;
END;
/

CREATE OR REPLACE PACKAGE qz_limb_mgr
   AUTHID DEFINER
IS
   TYPE qz_limb_nt1 IS TABLE OF qz_limb_ot;

   g_limbs_as_objects   qz_limb_nt1;

   TYPE qz_limb_nt2 IS TABLE OF qz_limbs%ROWTYPE;

   g_limbs_as_rows      qz_limb_nt2;
END;
/

CREATE OR REPLACE PACKAGE BODY qz_limb_mgr
IS
BEGIN
     SELECT qz_limb_ot (l.nm, l.avg_len)
       BULK COLLECT INTO g_limbs_as_objects
       FROM qz_limbs l
   ORDER BY l.nm;

     SELECT l.nm, l.avg_len
       BULK COLLECT INTO g_limbs_as_rows
       FROM qz_limbs l
   ORDER BY l.nm;
END;
/

DECLARE
   l_limbs   qz_limb_mgr.qz_limb_nt1;
BEGIN
   l_limbs :=
      qz_limb_mgr.g_limbs_as_objects
         MULTISET UNION qz_limb_mgr.g_limbs_as_objects;
   DBMS_OUTPUT.put_line ('Lots of limbs! ' || l_limbs.COUNT);
END;
/

DECLARE
   l_limbs   qz_limb_mgr.qz_limb_nt1;
BEGIN
   l_limbs :=
      qz_limb_mgr.g_limbs_as_objects
         MULTISET EXCEPT qz_limb_mgr.g_limbs_as_objects;
   DBMS_OUTPUT.put_line ('Lots of limbs! ' || l_limbs.COUNT);
END;
/

DECLARE
   l_limbs   qz_limb_mgr.qz_limb_nt1;
BEGIN
   l_limbs :=
      qz_limb_mgr.g_limbs_as_objects
         MULTISET UNION DISTINCT qz_limb_mgr.g_limbs_as_objects;
   DBMS_OUTPUT.put_line ('Lots of limbs! ' || l_limbs.COUNT);
END;
/

DECLARE
   l_limbs   qz_limb_mgr.qz_limb_nt2;
BEGIN
   l_limbs :=
      qz_limb_mgr.g_limbs_as_rows MULTISET UNION qz_limb_mgr.g_limbs_as_rows;
   DBMS_OUTPUT.put_line ('Lots of limbs! ' || l_limbs.COUNT);
END;
/

DECLARE
   l_limbs   qz_limb_mgr.qz_limb_nt1;
BEGIN
   l_limbs :=
      qz_limb_mgr.g_limbs_as_rows MULTISET EXCEPT qz_limb_mgr.g_limbs_as_rows;
   DBMS_OUTPUT.put_line ('Lots of limbs! ' || l_limbs.COUNT);
END;
/

DECLARE
   l_limbs   qz_limb_mgr.qz_limb_nt1;
BEGIN
   l_limbs :=
      qz_limb_mgr.g_limbs_as_objects
         MULTISET UNION DISTINCT qz_limb_mgr.g_limbs_as_objects;
   DBMS_OUTPUT.put_line ('Lots of limbs! ' || l_limbs.COUNT);
END;
/