CREATE OR REPLACE FUNCTION pluralize (
   number_in               IN NUMBER,
   label_in                IN VARCHAR2,
   plural_suffix_in        IN VARCHAR2 DEFAULT 's',
   plural_replacement_in   IN VARCHAR2 DEFAULT NULL)
   RETURN VARCHAR2
IS
BEGIN
   IF number_in IS NULL
   THEN
      RETURN NULL;
   ELSE
      RETURN    TO_CHAR (number_in)
             || ' '
             || CASE
                   WHEN plural_replacement_in IS NOT NULL
                   THEN
                      CASE number_in
                         WHEN 1 THEN label_in
                         ELSE plural_replacement_in
                      END
                   ELSE
                         label_in
                      || CASE number_in
                            WHEN 1 THEN NULL
                            ELSE plural_suffix_in
                         END
                END;
   END IF;
END pluralize;
/

BEGIN
   DBMS_OUTPUT.put_line (pluralize (1, 'Quiz'));
   DBMS_OUTPUT.put_line (pluralize (2, 'Quiz'));
   DBMS_OUTPUT.put_line (pluralize (15, 'Quiz', plural_suffix_in => 'zes'));
   DBMS_OUTPUT.put_line (pluralize (100, 'Country'));
   DBMS_OUTPUT.put_line (
      pluralize (100, 'Country', plural_suffix_in => 'ies'));
   DBMS_OUTPUT.put_line (
      pluralize (100, 'Country', plural_replacement_in => 'Countries'));
   DBMS_OUTPUT.put_line (
      pluralize (1, 'Country', plural_replacement_in => 'Countries'));
END;
/